;;; gitmail.el --- A similiar tool to git-send-email

;; Copyright (C) 2010  Leo

;; Author: Leo <sdl.web@gmail.com>
;; Keywords: tools, convenience, mail

;; This program is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation, either version 3 of the License, or
;; (at your option) any later version.

;; This program is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.

;;; Commentary:

;; 

;;; Code:

(eval-when-compile (require 'cl))
(require 'message)

;; (defvar message-unix-mail-delimiter)	; quiet compiler
(autoload 'gnus-alive-p "gnus-util")
(autoload 'dired-get-marked-files "dired")

(declare-function nndraft-request-associate-buffer "nndraft")
(declare-function gnus-agent-queue-setup "gnus-agent")
;; (declare-function message-fetch-field "message")
;; (declare-function message-add-header "message")
;; (declare-function message-send "message")
;; (declare-function message-narrow-to-headers-or-head "message")

(defvar gitmail-read-email-map (make-sparse-keymap)
  "Minibuffer keymap for `gitmail-read-email'.")

(set-keymap-parent gitmail-read-email-map minibuffer-local-map)

(when (fboundp 'bbdb-complete-name)
  (define-key gitmail-read-email-map [tab] 'bbdb-complete-name))

(defsubst gitmail-read-email (prompt)
  (read-from-minibuffer prompt nil gitmail-read-email-map))

(defun gitmail-map-over-mbox (function)
  "Call FUNCTION on each mbox email in current buffer."
  (save-excursion
    (goto-char (point-min))
    (let ((beg (re-search-forward message-unix-mail-delimiter nil t))
          end email stop)
      (when beg
        (while (not stop)
          (if (re-search-forward message-unix-mail-delimiter nil 'move)
              (setq end (match-beginning 0))
            (setq end (point))
            (setq stop t))
          (setq email (buffer-substring beg end))
          (with-temp-buffer
            (insert email)
            (goto-char (point-min))
            (re-search-forward "^$")
            (insert mail-header-separator)
            (goto-char (point-min))
            (funcall function))
          (setq beg (point)))))))

(defun gitmail-map-over-mbox-files (function files)
  "Call FUNCTION on each email in mbox FILES."
  (dolist (f files)
    (with-temp-buffer
      (insert-file-contents f)
      (gitmail-map-over-mbox function))))

(defun gitmail-import-mbox-1 ()
  (declare (special to-address cc-address))
  (message-mode)
  (message-narrow-to-headers-or-head)
  (message-add-header "X-Draft-Keep-Date: Yes")
  (nndraft-request-associate-buffer "drafts")
  (unless (message-fetch-field "to")
    (unless to-address
      (setq to-address (gitmail-read-email "To: ")))
    (unless (string-match "[ \t]*" to-address)
      (message-add-header (format "To: %s" to-address))))
  (unless (or (message-fetch-field "cc") (eq cc-address 'no))
    (unless cc-address
      (setq cc-address
            (if (yes-or-no-p "Add Cc addresses? ")
                (gitmail-read-email "Cc: ")
              'no)))
    (unless (eq cc-address 'no)
      (message-add-header (format "Cc: %s" cc-address))))
  (save-buffer 0))

(defun gitmail-send-mbox-1 ()
  (declare (special to-address cc-address))
  (message-mode)
  (message-narrow-to-headers-or-head)
  (unless (message-fetch-field "to")
    (unless to-address
      (setq to-address (gitmail-read-email "To address: ")))
    (message-add-header (format "To: %s" to-address)))
  (unless (or (message-fetch-field "cc") (eq cc-address 'no))
    (unless cc-address
      (setq cc-address
            (if (yes-or-no-p "Add Cc addresses? ")
                (gitmail-read-email "Cc: ")
              'no)))
    (unless (eq cc-address 'no)
      (message-add-header (format "Cc: %s" cc-address))))
  (message-send))

;;;###autoload
(defun gitmail-import-mbox-as-draft (&rest mbox-files)
  "Import emails in MBOX-FILES into the draft group."
  (interactive (if (derived-mode-p 'dired)
                   (dired-get-marked-files)
                 (file-expand-wildcards (read-file-name "Find mbox file: ") t)))
  (assert (gnus-alive-p) nil "Gnus is not running")
  (gnus-agent-queue-setup "drafts")
  (let (to-address cc-address)
    (gitmail-map-over-mbox-files 'gitmail-import-mbox-1 mbox-files)))

;;;###autoload
(defun gitmail-send-mbox (&rest mbox-files)
  "Send all emails in MBOX-FILES."
  (interactive (if (derived-mode-p 'dired)
                   (dired-get-marked-files)
                 (file-expand-wildcards (read-file-name "Find mbox file: ") t)))
  (let (to-address cc-address)
    (gitmail-map-over-mbox-files 'gitmail-send-mbox-1 mbox-files)))

;;;###autoload
(defun gitmail-send-mbox-buffer (&optional buffer)
  "Send mbox emails in BUFFER (default current buffer)."
  (interactive)
  (let (to-address cc-address)
    (with-current-buffer (or buffer (current-buffer))
      (gitmail-map-over-mbox 'gitmail-send-mbox-1))))

;;;###autoload
(defun gitmail-import-mbox-buffer (&optional buffer)
  "Send mbox emails in BUFFER (default current buffer)."
  (interactive)
  (assert (gnus-alive-p) nil "Gnus is not running")
  (let (to-address cc-address)
    (with-current-buffer (or buffer (current-buffer))
      (gitmail-map-over-mbox 'gitmail-import-mbox-1))))

(provide 'gitmail)
;;; gitmail.el ends here
